Правила GIT
===

## Основное правило

Ветку __master__ всегда можно развернуть. Всегда.

Развёртываемая (стабильная) ветка __master__ важна по нескольким причинам. 
Первым делом, это позволяет любому новичку на проекте выкачать и собрать проект без ошибок. 
Ничто так не разочаровывает, как невозможность собрать проект, с которым не знаком.

Во-вторых, __master__ показывает текущее состояние проекта в производстве. 
Если нужно сделать правки - всегда ясно, от чего надо ответвляться.

И, наконец, разворачиваемый __master__ - это страховочная сеть. 
Если __master__ развёртываем - то мы в любой момент можем развернуть проект 
из ветки без опасений, что что-то отвалится.

## Стратегии ветвления

Ветка __develop__ должна быть основной веткой, в которой ведётся разработка. 
Ветки для внедрения новых возможностей должны ответвляться от этой ветки, 
и вливаться обратно в ветку __develop__, 
а ветка __develop__ представляет состояние кодовой базы проекта на текущий момент.

Так как обе ветки - __master__ и __develop__ - являются постоянными, 
и постоянно затрагиваемыми, то работа с ними никогда не должна вестись напрямую. 

Вместо этого, вся работа должна вестись в отдельных ветках. 

Для начала работы, клонируем ветку __develop__.

При разработке новой функциональности необходимо ответвиться от ветки __develop__, 
и разработку вести во вновь созданной ветке.

## Отдельную задачу делаем в отдельной ветке

Другими словами, разделяйте реализацию задач так, 
чтобы неперекающийся функционал был в разных ветках.

Иногда нужно выложить на тест одну задачу, 
а с ней в этой же ветке находится другая задача, которая еще не завершена.
Бывает, что паралельная задача нарушает работу чего-либо из-за незавершенности...

## Как именовать ветки

Не существует жёстких правил именования веток, особенно веток, 
в которых ведётся доработка или разработка нового функционала. 

Давайте веткам понятные имена, чтобы прочитав имя, 
было понятно, с чем в этой ветке работает программист.

Если ветка создана __для правки__, то, возможно, неплохо дать ей окончание `-fix`. 

Например:

```
profile-fix
```

Если ветка создана __для новой функциональности__, то обычно так её и называют: 

```
profile-feature
```

В конце имени ветки можно добавлять ID выполняемой задачи.
Так будет легче соотносить ветки с задачами.

Например:

```
profile-fix-1234
```

## Обзор кода

Скажем, мы сделали ветку, понаписали кода, объединили/переместили изменения из __develop__, 
и вот мы готовы к заливке кода обратно в __develop__. 
Но должны ли? 
Может, кто-то для начала должен просмотреть наши изменения?

__Обзор кода__ - отличная вещь! 
Он позволяет вам получить ценные отзывы о той работе, что вы провели, 
а также повысить вероятность обнаружения и исправления ошибок.

Вот где на помощь приходят запросы на включение (__pull-request__). 

Когда дело доходит до запросов на включение, 
желательно чтобы вливание нового кода производилось тем же человеком, 
который открыл запрос на включение, так как обычно это тот же человек, 
который этот код написал. 
Чтобы этого добиться, рецензент должен оставить комментарий о том, 
что код прошёл рассмотрение, и замечаний нет, 
но не должен нажимать кнопку применения изменений. 
Как только коллега нажимает кнопку “Всё ОК” (говоря фигурально), 
персона, открывшая запрос на включение может объединить изменения с основной веткой.

## Поправь, пока не остыло

Мы не должны выпускать ошибки.

…но уж если это произошло, то лучше их поправить как можно быстрее. 
Так как ветка __develop__ может содержать незаконченные изменения, 
ветки критических правок надо создавать от текущего релиза, 
который находится в master (так как __master__ всегда служит 
для разворачивания для конечных пользователей).

Чтобы сделать правку, создайте ветку от __master__, сделайте правку, 
потом сделайте неперематываемое слияние (__non-fast-forward merge__) в __master__. 
Создайте тег, слейте __master__ в __develop__ (так как мы хотим, 
чтобы правка попала и в __develop__). 
Теперь можно удалить ветку с правкой.

## Время поговорить о коммитах

Давайте поговорим о сообщениях коммитов в __Git__. 
Намного проще будет просматривать историю, 
если разработчики придерживаются единого формата сообщений коммитов. 

Вот несколько хороших правил:

* Первая строка должна содержать краткую суть коммита 
(желательно до 50 символов).
* Если заглавная строка требует пояснения, можно, пропустив одну строку, 
написать дополнительное описание. 
Описание должно быть оформлено в виде параграфа, с правильным озаглавливанием и пунктуацией.
* Сообщения коммита необходимо разбивать на строки длиной до 72 символов, 
чтобы журнал коммитов нормально отображался в терминалах.
* комит должен быть атомарным
* комит должен содержать только нужное, внимательно просмаривайте изменения
* чистку кода и Code Style делаем отдельным комитом

## Чистка кода и Code Style

На первый взгяд ничего страшного в том, что мы где-то отформатируем код.
Но если в этом же файле кто-то внесет изменения, 
то отследить изменения при слиянии ветки будет гораздо сложней.

Чистку кода или Code Style можно делать, когда Вы уверены, 
что с текущим кодом работаете только вы.

## Комиты, как точки опоры

Делайте комиты, когда код работает кооректно.
Пусть код еще не идеален, но он работает.

Например, код работает и мы его закомитили.
Затем делаем рефокторинг кода.
В случае неудачного рефакторинга, когда Вы получили не рабочий код, 
есть возможность отката изменений к рабочему состоянию.

Комитим каждый шаг рефакторинга, чтобы зафиксировать рабочее состояние.
